@@ -27,13 +27,15 @@ Example:
 
 import ironflow.node_tools.input_widgets
 import ironflow.node_tools.main_widgets
-from ironflow.model import dtypes, NodeInputBP
+from ironflow.model import dtypes
 from ironflow.model.node import (
     Node,
     PlaceholderWidgetsContainer,
     PortList,
+    BatchingNode,
     DataNode,
     JobMaker,
+    JobNode,
     JobTaker,
 )
-from ironflow.model.port import NodeOutputBP
+from ironflow.model.port import NodeInputBP, NodeOutputBP
